/*
 * This file is part of the Computer Vision Toolkit (cvkit).
 *
 * Author: Heiko Hirschmueller
 *
 * Copyright (c) 2014, Institute of Robotics and Mechatronics, German Aerospace Center
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.
 *
 * 3. Neither the name of the copyright holder nor the names of its contributors
 * may be used to endorse or promote products derived from this software without
 * specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 */

#ifndef GVR_GLMAIN_H
#define GVR_GLMAIN_H

namespace gvr
{

class GLListener
{
  public:

    virtual void onRedraw() { };
    virtual void onReshape(int w, int h) { };
    virtual void onSpecialKey(int key, int x, int y) { };
    virtual void onKey(unsigned char key, int x, int y) { };
    virtual void onMouseButton(int button, int state, int x, int y) { };
    virtual void onMouseMove(int x, int y) { };
};

/**
 * Initialization. Please make sure that this function is called first in the
 * program.
 */

void GLInit(int &argc, char **argv);

/**
 * Creates a window and initializes it for drawing with a depth buffer.
 */

void GLInitWindow(int x, int y, int w, int h, const char *title);

/**
 * Mark window for redisplay.
 */

void GLRedisplay();

/**
 * Register a callback that is called (once) in the given amount of
 * milliseconds.
 */

void GLTimerFunc(unsigned int milliseconds, void (*fct)(int value), int value);

/**
 * Setting the Listener for the event callbacks and enter the GLUT main loop.
 * This function will never return!
 */

void GLMainLoop(GLListener &listener);

/**
  Leaves main loop.
*/

void GLLeaveMainLoop();

}

#endif
